@@ -118,18 +118,16 @@ typedef struct _VBOXMP_DEVEXT
    volatile uint32_t cContexts3D;
    volatile uint32_t cContexts2D;
    volatile uint32_t cContextsDispIfResize;
-   volatile uint32_t cRenderFromShadowDisabledContexts;
    volatile uint32_t cUnlockedVBVADisabled;
 
    volatile uint32_t fCompletingCommands;
 
    DWORD dwDrvCfgFlags;
-   /* this is examined and swicthed by DxgkDdiSubmitCommand only! */
-   volatile BOOLEAN fRenderToShadowDisabled;
-
+#ifdef VBOX_WITH_CROGL
    BOOLEAN f3DEnabled;
    BOOLEAN fTexPresentEnabled;
    BOOLEAN fCmdVbvaEnabled;
+   BOOLEAN fComplexTopologiesEnabled;
 
    uint32_t u32CrConDefaultClientID;
 
@@ -137,7 +135,7 @@ typedef struct _VBOXMP_DEVEXT
 
    VBOXMP_CRCTLCON CrCtlCon;
    VBOXMP_CRSHGSMITRANSPORT CrHgsmiTransport;
-
+#endif
    VBOXWDDM_GLOBAL_POINTER_INFO PointerInfo;
 
    VBOXVTLIST CtlList;
@@ -145,7 +143,11 @@ typedef struct _VBOXMP_DEVEXT
 #ifdef VBOX_WITH_VIDEOHWACCEL
    VBOXVTLIST VhwaCmdList;
 #endif
-   BOOL bNotifyDxDpc;
+   BOOLEAN bNotifyDxDpc;
+
+   BOOLEAN fDisableTargetUpdate;
+
+
 
 #ifdef VBOX_VDMA_WITH_WATCHDOG
    PKTHREAD pWdThread;
@@ -219,6 +221,7 @@ DECLINLINE(PVBOXMP_COMMON) VBoxCommonFromDeviceExt(PVBOXMP_DEVEXT pExt)
 #ifdef VBOX_WDDM_MINIPORT
 DECLINLINE(ULONG) vboxWddmVramCpuVisibleSize(PVBOXMP_DEVEXT pDevExt)
 {
+#ifdef VBOX_WITH_CROGL
     if (pDevExt->fCmdVbvaEnabled)
     {
         /* all memory layout info should be initialized */
@@ -228,6 +231,7 @@ DECLINLINE(ULONG) vboxWddmVramCpuVisibleSize(PVBOXMP_DEVEXT pDevExt)
 
         return (ULONG)(pDevExt->CmdVbva.Vbva.offVRAMBuffer & ~0xfffULL);
     }
+#endif
     /* all memory layout info should be initialized */
     Assert(pDevExt->aSources[0].Vbva.Vbva.offVRAMBuffer);
     /* page aligned */
@@ -241,12 +245,14 @@ DECLINLINE(ULONG) vboxWddmVramCpuVisibleSegmentSize(PVBOXMP_DEVEXT pDevExt)
     return vboxWddmVramCpuVisibleSize(pDevExt);
 }
 
-#ifdef VBOXWDDM_RENDER_FROM_SHADOW
+/* 128 MB */
 DECLINLINE(ULONG) vboxWddmVramCpuInvisibleSegmentSize(PVBOXMP_DEVEXT pDevExt)
 {
-    return vboxWddmVramCpuVisibleSegmentSize(pDevExt);
+    return 128 * 1024 * 1024;
 }
 
+#ifdef VBOXWDDM_RENDER_FROM_SHADOW
+
 DECLINLINE(bool) vboxWddmCmpSurfDescsBase(VBOXWDDM_SURFACE_DESC *pDesc1, VBOXWDDM_SURFACE_DESC *pDesc2)
 {
     if (pDesc1->width != pDesc2->width)
