@@ -537,6 +537,12 @@ int vboxVhwaHlpPopulateSurInfo(VBOXVHWA_SURFACEDESC *pInfo, PVBOXWDDM_ALLOCATION
 {
     memset(pInfo, 0, sizeof(VBOXVHWA_SURFACEDESC));
 
+    if (pSurf->AllocData.Addr.SegmentId != 1)
+    {
+        WARN(("invalid segment id!"));
+        return VERR_INVALID_PARAMETER;
+    }
+
     pInfo->height = pSurf->AllocData.SurfDesc.height;
     pInfo->width = pSurf->AllocData.SurfDesc.width;
     pInfo->flags |= VBOXVHWA_SD_HEIGHT | VBOXVHWA_SD_WIDTH;
@@ -813,6 +819,13 @@ int vboxVhwaHlpOverlayFlip(PVBOXWDDM_OVERLAY pOverlay, const DXGKARG_FLIPOVERLAY
     Assert(pOverlay->pCurentAlloc->pResource == pOverlay->pResource);
     Assert(pOverlay->pCurentAlloc != pAlloc);
     int rc = VINF_SUCCESS;
+
+    if (pFbSurf->AllocData.Addr.SegmentId != 1)
+    {
+        WARN(("invalid segment id on flip"));
+        return VERR_INVALID_PARAMETER;
+    }
+
     if (pFlipInfo->PrivateDriverDataSize == sizeof (VBOXWDDM_OVERLAYFLIP_INFO))
     {
         PVBOXWDDM_OVERLAYFLIP_INFO pOurInfo = (PVBOXWDDM_OVERLAYFLIP_INFO)pFlipInfo->pPrivateDriverData;
@@ -876,18 +889,13 @@ int vboxVhwaHlpColorFill(PVBOXWDDM_OVERLAY pOverlay, PVBOXWDDM_DMA_PRIVATEDATA_C
 {
     PVBOXWDDM_ALLOCATION pAlloc = pCF->ClrFill.Alloc.pAlloc;
     Assert(pAlloc->pResource == pOverlay->pResource);
-#ifdef VBOXWDDM_RENDER_FROM_SHADOW
-    if (pAlloc->bAssigned)
+
+    if (pAlloc->AllocData.Addr.SegmentId != 1)
     {
-        /* check if this is a primary surf */
-        PVBOXWDDM_SOURCE pSource = &pOverlay->pDevExt->aSources[pOverlay->VidPnSourceId];
-        if (pSource->pPrimaryAllocation == pAlloc)
-        {
-            pAlloc = pSource->pShadowAllocation;
-            Assert(pAlloc->pResource == pOverlay->pResource);
-        }
+        WARN(("invalid segment id on color fill"));
+        return VERR_INVALID_PARAMETER;
     }
-#endif
+
     Assert(pAlloc->hHostHandle);
     Assert(pAlloc->pResource);
     Assert(pAlloc->AllocData.Addr.offVram != VBOXVIDEOOFFSET_VOID);
@@ -970,6 +978,13 @@ int vboxVhwaHlpOverlayUpdate(PVBOXWDDM_OVERLAY pOverlay, const DXGK_OVERLAYINFO
     Assert(pFbSurf->hHostHandle);
     Assert(pFbSurf->AllocData.Addr.offVram != VBOXVIDEOOFFSET_VOID);
     int rc = VINF_SUCCESS;
+
+    if (pFbSurf->AllocData.Addr.SegmentId != 1)
+    {
+        WARN(("invalid segment id on overlay update"));
+        return VERR_INVALID_PARAMETER;
+    }
+
     if (pOverlayInfo->PrivateDriverDataSize == sizeof (VBOXWDDM_OVERLAY_INFO))
     {
         PVBOXWDDM_OVERLAY_INFO pOurInfo = (PVBOXWDDM_OVERLAY_INFO)pOverlayInfo->pPrivateDriverData;
@@ -1107,14 +1122,10 @@ int vboxVhwaHlpOverlayCreate(PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE
             pOverlay->VidPnSourceId = VidPnSourceId;
 
             vboxVhwaHlpOverlayListAdd(pDevExt, pOverlay);
-#ifdef VBOXWDDM_RENDER_FROM_SHADOW
+
             RECT DstRect;
             vboxVhwaHlpOverlayDstRectGet(pDevExt, pOverlay, &DstRect);
-            NTSTATUS Status = vboxVdmaHlpUpdatePrimary(pDevExt, VidPnSourceId, &DstRect);
-            Assert(Status == STATUS_SUCCESS);
-            /* ignore primary update failure */
-            Status = STATUS_SUCCESS;
-#endif
+
             rc = vboxVhwaHlpOverlayUpdate(pOverlay, pOverlayInfo, DstRect.right ? &DstRect : NULL);
             if (!RT_SUCCESS(rc))
             {
