@@ -878,7 +878,7 @@ void UISelectorWindow::sltCurrentVMItemChanged(bool fRefreshDetails, bool fRefre
                    "at the top of the window.</p>"
                    "<p>You can press the <b>%1</b> key to get instant help, "
                    "or visit "
-                   "<a href=http://www.virtualbox.org>www.virtualbox.org</a> "
+                   "<a href=https://www.virtualbox.org>www.virtualbox.org</a> "
                    "for the latest information and news.</p>")
                    .arg(QKeySequence(QKeySequence::HelpContents).toString(QKeySequence::NativeText)));
         }
