@@ -1,10 +1,10 @@
-/* $Revision: 80003 $ */
+/* $Id: SUPDrvInternal.h $ */
 /** @file
  * VirtualBox Support Driver - Internal header.
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -58,7 +58,7 @@
 #       define _interlockedbittestandset64    _interlockedbittestandset64_StupidDDKVsCompilerCrap
 #       define _interlockedbittestandreset64  _interlockedbittestandreset64_StupidDDKVsCompilerCrap
 #       pragma warning(disable : 4163)
-#       include <ntddk.h>
+#       include <iprt/nt/nt.h>
 #       pragma warning(default : 4163)
 #       undef  _InterlockedExchange
 #       undef  _InterlockedExchangeAdd
@@ -69,11 +69,9 @@
 #       undef  _interlockedbittestandset64
 #       undef  _interlockedbittestandreset64
 #   else
-#       include <ntddk.h>
+#       include <iprt/nt/nt.h>
 #   endif
 #   include <memory.h>
-#   define memcmp(a,b,c) mymemcmp(a,b,c)
-    int VBOXCALL mymemcmp(const void *, const void *, size_t);
     RT_C_DECLS_END
 
 #elif defined(RT_OS_LINUX)
@@ -177,6 +175,13 @@
  */
 #define OSDBGPRINT(a) SUPR0Printf a
 
+/** Debug printf macro shared with the ring-3 part. */
+#ifdef DEBUG_bird
+# define SUP_DPRINTF(a) SUPR0Printf a
+#else
+# define SUP_DPRINTF(a) do { } while (0)
+#endif
+
 
 /** @name Context values for the per-session handle tables.
  * The context value is used to distinguish between the different kinds of
@@ -199,6 +204,16 @@
     (   VALID_PTR(pSession) \
      && pSession->u32Cookie == BIRD_INV)
 
+/**
+ * Validates a device extension pointer.
+ *
+ * @returns true/false accordingly.
+ * @param   pDevExt     The device extension.
+ */
+#define SUP_IS_DEVEXT_VALID(pDevExt) \
+    (   VALID_PTR(pDevExt)\
+     && pDevExt->u32Cookie == BIRD)
+
 
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
@@ -378,9 +393,9 @@ typedef struct SUPDRVOBJ
 } SUPDRVOBJ, *PSUPDRVOBJ;
 
 /** Magic number for SUPDRVOBJ::u32Magic. (Dame Agatha Mary Clarissa Christie). */
-#define SUPDRVOBJ_MAGIC             0x18900915
+#define SUPDRVOBJ_MAGIC             UINT32_C(0x18900915)
 /** Dead number magic for SUPDRVOBJ::u32Magic. */
-#define SUPDRVOBJ_MAGIC_DEAD        0x19760112
+#define SUPDRVOBJ_MAGIC_DEAD        UINT32_C(0x19760112)
 
 /**
  * The per-session object usage record.
@@ -406,16 +421,38 @@ typedef struct SUPDRVSESSION
     PSUPDRVDEVEXT                   pDevExt;
     /** Session Cookie. */
     uint32_t                        u32Cookie;
+    /** Set if is an unrestricted session, clear if restricted. */
+    bool                            fUnrestricted;
+
+    /** Set if we're in the hash table, clear if not.  Protected by the hash
+     * table spinlock. */
+    bool                            fInHashTable;
+    /** Reference counter. */
+    uint32_t volatile               cRefs;
+    /** Pointer to the next session with the same hash (common hash table).
+     *  Protected by the hash table spinlock. */
+    PSUPDRVSESSION                  pCommonNextHash;
+    /** Pointer to the OS specific session pointer, if available and in use.
+     * This is atomically set and cleared as the session is inserted and removed
+     * from the hash table (protected by the session hash table spinlock). */
+    PSUPDRVSESSION                 *ppOsSessionPtr;
+    /** The process (id) of the session. */
+    RTPROCESS                       Process;
+    /** Which process this session is associated with.
+     * This is NIL_RTR0PROCESS for kernel sessions and valid for user ones. */
+    RTR0PROCESS                     R0Process;
 
     /** The VM associated with the session. */
     PVM                             pVM;
     /** Handle table for IPRT semaphore wrapper APIs.
-     * Programmable from R0 and R3. */
+     * This takes care of its own locking in an IRQ safe manner. */
     RTHANDLETABLE                   hHandleTable;
     /** Load usage records. (protected by SUPDRVDEVEXT::mtxLdr) */
     PSUPDRVLDRUSAGE volatile        pLdrUsage;
 
-    /** Spinlock protecting the bundles and the GIP members. */
+    /** Spinlock protecting the bundles, the GIP members and the
+     * fProcessCleanupDone flag.  It continues to be valid until the last
+     * reference to the session is released. */
     RTSPINLOCK                      Spinlock;
     /** The ring-3 mapping of the GIP (readonly). */
     RTR0MEMOBJ                      GipMapObjR3;
@@ -430,11 +467,6 @@ typedef struct SUPDRVSESSION
     RTUID                           Uid;
     /** The group id of the session. (Set by the OS part.) */
     RTGID                           Gid;
-    /** The process (id) of the session. */
-    RTPROCESS                       Process;
-    /** Which process this session is associated with.
-     * This is NIL_RTR0PROCESS for kernel sessions and valid for user ones. */
-    RTR0PROCESS                     R0Process;
     /** Per session tracer specfic data. */
     uintptr_t                       uTracerData;
     /** The thread currently actively talking to the tracer. (One at the time!) */
@@ -467,6 +499,10 @@ typedef struct SUPDRVSESSION
     /** Pointer to the next session with the same hash. */
     PSUPDRVSESSION                  pNextHash;
 # endif
+# if defined(RT_OS_WINDOWS) && defined(VBOX_WITH_HARDENING)
+    /** Pointer to the process protection structure for this session. */
+    struct SUPDRVNTPROTECT         *pNtProtect;
+# endif
 #endif /* !SUPDRV_AGNOSTIC */
 } SUPDRVSESSION;
 
@@ -476,7 +512,15 @@ typedef struct SUPDRVSESSION
  */
 typedef struct SUPDRVDEVEXT
 {
-    /** Spinlock to serialize the initialization, usage counting and objects. */
+    /** Global cookie. */
+    uint32_t                        u32Cookie;
+    /** The actual size of SUPDRVSESSION. (SUPDRV_AGNOSTIC) */
+    uint32_t                        cbSession;
+
+    /** Spinlock to serialize the initialization, usage counting and objects.
+     * This is IRQ safe because we want to be able signal semaphores from the
+     * special HM context (and later maybe interrupt handlers), so we must be able
+     * to reference and dereference handles when IRQs are disabled. */
     RTSPINLOCK                      Spinlock;
 
     /** List of registered objects. Protected by the spinlock. */
@@ -484,11 +528,6 @@ typedef struct SUPDRVDEVEXT
     /** List of free object usage records. */
     PSUPDRVUSAGE volatile           pUsageFree;
 
-    /** Global cookie. */
-    uint32_t                        u32Cookie;
-    /** The actual size of SUPDRVSESSION. (SUPDRV_AGNOSTIC) */
-    uint32_t                        cbSession;
-
     /** Loader mutex.
      * This protects pvVMMR0, pvVMMR0Entry, pImages and SUPDRVSESSION::pLdrUsage. */
 #ifdef SUPDRV_USE_MUTEX_FOR_LDR
@@ -509,6 +548,8 @@ typedef struct SUPDRVDEVEXT
 
     /** Linked list of loaded code. */
     PSUPDRVLDRIMAGE volatile        pLdrImages;
+    /** Set if the image loading interface got disabled after loading all needed images */
+    bool                            fLdrLockedDown;
 
     /** @name These members for detecting whether an API caller is in ModuleInit.
      * Certain APIs are only permitted from ModuleInit, like for instance tracepoint
@@ -529,7 +570,9 @@ typedef struct SUPDRVDEVEXT
 #else
     RTSEMFASTMUTEX                  mtxGip;
 #endif
-    /** GIP spinlock protecting GIP members during Mp events. */
+    /** GIP spinlock protecting GIP members during Mp events.
+     * This is IRQ safe since be may get MP callbacks in contexts where IRQs are
+     * disabled (on some platforms). */
     RTSPINLOCK                      hGipSpinlock;
     /** Pointer to the Global Info Page (GIP). */
     PSUPGLOBALINFOPAGE              pGip;
@@ -577,8 +620,21 @@ typedef struct SUPDRVDEVEXT
     /** Hash table for user tracer modules (SUPDRVVTGCOPY). */
     RTLISTANCHOR                    aTrackerUmodHash[128];
 
+    /** @name Session Handle Table.
+     * @{ */
+    /** Spinlock protecting apSessionHashTab, cSessions,
+     * SUPDRVSESSION::ppOsSessionPtr, SUPDRVSESSION::pCommonNextHash, and possibly
+     * others depending on the OS. */
+    RTSPINLOCK                      hSessionHashTabSpinlock;
+    /** Session hash table hash table.  The size of this table must make sense in
+     * comparison to GVMM_MAX_HANDLES. */
+    PSUPDRVSESSION                  apSessionHashTab[HC_ARCH_BITS == 64 ? 8191 : 127];
+    /** The number of open sessions. */
+    int32_t                         cSessions;
+    /** @} */
+
     /*
-     * Note! The non-agnostic bits must be a the very end of the structure!
+     * Note! The non-agnostic bits must be at the very end of the structure!
      */
 #ifndef SUPDRV_AGNOSTIC
 # ifdef RT_OS_WINDOWS
@@ -590,16 +646,50 @@ typedef struct SUPDRVDEVEXT
 #endif
 } SUPDRVDEVEXT;
 
+/** Calculates the index into g_apSessionHashTab.*/
+#define SUPDRV_SESSION_HASH(a_pid)     ( (a_pid) % RT_ELEMENTS(((SUPDRVDEVEXT *)NULL)->apSessionHashTab) )
+
 
 RT_C_DECLS_BEGIN
 
 /*******************************************************************************
 *   OS Specific Functions                                                      *
 *******************************************************************************/
+/**
+ * Called to clean up the session structure before it's freed.
+ *
+ * @param   pDevExt             The device globals.
+ * @param   pSession            The session that's being cleaned up.
+ */
+void VBOXCALL   supdrvOSCleanupSession(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession);
+
+/**
+ * Called to let the OS specfic code perform additional insertion work while
+ * still under the protection of the hash table spinlock.
+ *
+ * @param   pDevExt             The device globals.
+ * @param   pSession            The session that was inserted.
+ * @param   pvUser              User context specified to the insert call.
+ */
+void VBOXCALL   supdrvOSSessionHashTabInserted(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, void *pvUser);
+
+/**
+ * Called to let the OS specfic code perform additional removal work while still
+ * under the protection of the hash table spinlock.
+ *
+ * @param   pDevExt             The device globals.
+ * @param   pSession            The session that was removed.
+ * @param   pvUser              User context specified to the remove call.
+ */
+void VBOXCALL   supdrvOSSessionHashTabRemoved(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, void *pvUser);
+
 void VBOXCALL   supdrvOSObjInitCreator(PSUPDRVOBJ pObj, PSUPDRVSESSION pSession);
 bool VBOXCALL   supdrvOSObjCanAccess(PSUPDRVOBJ pObj, PSUPDRVSESSION pSession, const char *pszObjName, int *prc);
 bool VBOXCALL   supdrvOSGetForcedAsyncTscMode(PSUPDRVDEVEXT pDevExt);
 int  VBOXCALL   supdrvOSEnableVTx(bool fEnabled);
+RTCCUINTREG VBOXCALL supdrvOSChangeCR4(RTCCUINTREG fOrMask, RTCCUINTREG fAndMask);
+bool VBOXCALL   supdrvOSSuspendVTxOnCpu(void);
+void VBOXCALL   supdrvOSResumeVTxOnCpu(bool fSuspended);
 
 /**
  * Try open the image using the native loader.
@@ -663,18 +753,27 @@ int  VBOXCALL   supdrvOSLdrLoad(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage, c
 void VBOXCALL   supdrvOSLdrUnload(PSUPDRVDEVEXT pDevExt, PSUPDRVLDRIMAGE pImage);
 
 
+#if defined(RT_OS_DARWIN)
+int VBOXCALL    supdrvDarwinResumeSuspendedKbds(void);
+#endif
+
+
 /*******************************************************************************
 *   Shared Functions                                                           *
 *******************************************************************************/
 /* SUPDrv.c */
-int  VBOXCALL   supdrvIOCtl(uintptr_t uIOCtl, PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, PSUPREQHDR pReqHdr);
+int  VBOXCALL   supdrvIOCtl(uintptr_t uIOCtl, PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, PSUPREQHDR pReqHdr, size_t cbReq);
 int  VBOXCALL   supdrvIOCtlFast(uintptr_t uIOCtl, VMCPUID idCpu, PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession);
 int  VBOXCALL   supdrvIDC(uintptr_t uIOCtl, PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, PSUPDRVIDCREQHDR pReqHdr);
 int  VBOXCALL   supdrvInitDevExt(PSUPDRVDEVEXT pDevExt, size_t cbSession);
 void VBOXCALL   supdrvDeleteDevExt(PSUPDRVDEVEXT pDevExt);
-int  VBOXCALL   supdrvCreateSession(PSUPDRVDEVEXT pDevExt, bool fUser, PSUPDRVSESSION *ppSession);
-void VBOXCALL   supdrvCloseSession(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession);
-void VBOXCALL   supdrvCleanupSession(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession);
+int  VBOXCALL   supdrvCreateSession(PSUPDRVDEVEXT pDevExt, bool fUser, bool fUnrestricted,  PSUPDRVSESSION *ppSession);
+int  VBOXCALL   supdrvSessionHashTabInsert(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, PSUPDRVSESSION *ppOsSessionPtr, void *pvUser);
+int  VBOXCALL   supdrvSessionHashTabRemove(PSUPDRVDEVEXT pDevExt, PSUPDRVSESSION pSession, void *pvUser);
+PSUPDRVSESSION VBOXCALL supdrvSessionHashTabLookup(PSUPDRVDEVEXT pDevExt, RTPROCESS Process, RTR0PROCESS R0Process,
+                                                   PSUPDRVSESSION *ppOsSessionPtr);
+uint32_t VBOXCALL supdrvSessionRetain(PSUPDRVSESSION pSession);
+uint32_t VBOXCALL supdrvSessionRelease(PSUPDRVSESSION pSession);
 
 int  VBOXCALL   supdrvTracerInit(PSUPDRVDEVEXT pDevExt);
 void VBOXCALL   supdrvTracerTerm(PSUPDRVDEVEXT pDevExt);
